package fsmNetworkExample;

/**
 * The simplest finite state machine: an alarm clock.
 * It accepts just CLK_PULSE input (simulates a pulse from hardware pulse generator)
 * The internal timer counts up from zero until the number "target" is reached.
 * Most of the time, the FSM's Mealy output is QUIET.
 * Only "the last" CLK_PULSE that leads to "target" generates "BEEP" Mealy output.
 * 
 * FSM's Moore output is just a string with state number.
 * 
 * Example FSM for target = 3:
 * 
 *    CLK_PULSE/      CLK_PULSE/    CLK_PULSE/    CLK_PULSE/
 *    QUIET           QUIET         BEEP          QUIET
 * 0  ----------> 1  ----------> 2  ----------> 3 --------+
 *                                              ^         |
 *                                              |         |
 *                                              +---------+  
 *  
 */
public class FSMSimpleAlarmClock implements FSM {
	private int state;
	private int target;
	
	final public static int IN_CLK_PULSE = -1;
	final public static int OUT_QUIET = 0;
	final public static int OUT_BEEP = 1;

	public FSMSimpleAlarmClock(int target) {
		state = 0;
		this.target = target;
	}

	@Override
	public void reset() {
		state = 0;		
	}

	@Override
	public int push(int input) {
		if (input == IN_CLK_PULSE) {
			if (state == target - 1) {
				state = target;
				return OUT_BEEP;
			}
			else if (state == target) {
				return OUT_QUIET;
			}
			else {
				state++;
				return OUT_QUIET;
			}
		}
		return OUT_QUIET;
	}

	@Override
	public String getStateOutput() {
		return Integer.toString(state);
	}

}
